from traceback import format_exc
import os
import sys
import argparse
from llmengine.gemma3_it import Gemma3LLM
from llmengine.qwen3 import Qwen3LLM
from llmengine.medgemma3_it import MedgemmaLLM

from appPublic.registerfunction import RegisterFunction
from appPublic.log import debug
from ahserver.serverenv import ServerEnv
from ahserver.globalEnv import stream_response
from ahserver.webapp import webserver

from aiohttp_session import get_session
model_pathMap = {
	"/share/models/Qwen/Qwen3-32B": Qwen3LLM,
	"/share/models/google/gemma-3-4b-it": Gemma3LLM,
	"/share/models/google/medgemma-4b-it": MedgemmaLLM
}
def register(model_path, Klass):
	model_pathMap[model_path] = Klass

def init():
	rf = RegisterFunction()
	rf.register('chat_completions', chat_completions)

async def chat_completions(request, params_kw, *params, **kw):
	debug(f'{params_kw=}, {params=}, {kw=}')
	async def gor():
		se = ServerEnv()
		engine = se.chat_engine
		session = await get_session(request)
		kwargs = {
		}
		if params_kw.image_path:
			kwargs['image_path'] = fs.reapPath(params_kw.image_path)
		if params_kw.video_path:
			kwargs['video_path'] = fs.reapPath(params_kw.video_path)
		if params_kw.audio_path:
			kwargs['audio_path'] = fs.reapPath(params_kw.audio_path)
		async for d in engine.async_stream_generate(session, params_kw.prompt, **kwargs):
			debug(f'{d=}')
			yield d

	return await stream_response(request, gor)

def main():
	parser = argparse.ArgumentParser(prog="Sage")
	parser.add_argument('-w', '--workdir')
	parser.add_argument('-p', '--port')
	parser.add_argument('model_path')
	args = parser.parse_args()
	Klass =  model_pathMap.get(args.model_path)
	if Klass is None:
		e = Exception(f'{model_path} has not mapping to a model class')
		exception(f'{e}, {format_exc()}')
		raise e
	se = ServerEnv()
	se.chat_engine = Klass(args.model_path)
	workdir = args.workdir or os.getcwd()
	port = args.port 
	webserver(init, workdir, port)

if  __name__ == '__main__':
	main()

